@@ -1,7 +1,7 @@
 /*
  * server.h - Define shadowsocks server's buffers and callbacks
  *
- * Copyright (C) 2013 - 2016, Max Lv <max.c.lv@gmail.com>
+ * Copyright (C) 2013 - 2017, Max Lv <max.c.lv@gmail.com>
  *
  * This file is part of the shadowsocks-libev.
  *
@@ -23,11 +23,16 @@
 #ifndef _SERVER_H
 #define _SERVER_H
 
-#include <ev.h>
 #include <time.h>
 #include <libcork/ds.h>
 
-#include "encrypt.h"
+#ifdef HAVE_LIBEV_EV_H
+#include <libev/ev.h>
+#else
+#include <ev.h>
+#endif
+
+#include "crypto.h"
 #include "jconf.h"
 #include "resolv.h"
 
@@ -37,7 +42,6 @@ typedef struct listen_ctx {
     ev_io io;
     int fd;
     int timeout;
-    int method;
     char *iface;
     struct ev_loop *loop;
 } listen_ctx_t;
@@ -49,27 +53,42 @@ typedef struct server_ctx {
     struct server *server;
 } server_ctx_t;
 
+#ifdef USE_NFCONNTRACK_TOS
+
+#include <libnetfilter_conntrack/libnetfilter_conntrack.h>
+#include <libnetfilter_conntrack/libnetfilter_conntrack_tcp.h>
+
+struct dscptracker {
+        struct nf_conntrack *ct;
+        long unsigned int mark;
+        unsigned int dscp;
+        unsigned int packet_count;
+};
+
+#endif
+
+struct query;
+
 typedef struct server {
     int fd;
     int stage;
-    int auth;
     int frag;
 
     buffer_t *buf;
-    buffer_t *header_buf;
 
-    struct chunk *chunk;
-
-    struct enc_ctx *e_ctx;
-    struct enc_ctx *d_ctx;
+    cipher_ctx_t *e_ctx;
+    cipher_ctx_t *d_ctx;
     struct server_ctx *recv_ctx;
     struct server_ctx *send_ctx;
     struct listen_ctx *listen_ctx;
     struct remote *remote;
 
-    struct ResolvQuery *query;
+    struct query *query;
 
     struct cork_dllist_item entries;
+#ifdef USE_NFCONNTRACK_TOS
+    struct dscptracker* tracker;
+#endif
 } server_t;
 
 typedef struct query {
